# Code generated - EDITING IS FUTILE. DO NOT EDIT.

import typing
from ..cog import builder as cogbuilder
from ..models import googlecloudmonitoring
from ..models import dashboard


class CloudMonitoringQuery(cogbuilder.Builder[googlecloudmonitoring.CloudMonitoringQuery]):
    _internal: googlecloudmonitoring.CloudMonitoringQuery

    def __init__(self):
        self._internal = googlecloudmonitoring.CloudMonitoringQuery()

    def build(self) -> googlecloudmonitoring.CloudMonitoringQuery:
        """
        Builds the object.
        """
        return self._internal    
    
    def ref_id(self, ref_id: str) -> typing.Self:    
        """
        A unique identifier for the query within the list of targets.
        In server side expressions, the refId is used as a variable name to identify results.
        By default, the UI will assign A->Z; however setting meaningful names may be useful.
        """
            
        self._internal.ref_id = ref_id
    
        return self
    
    def hide(self, hide: bool) -> typing.Self:    
        """
        true if query is disabled (ie should not be returned to the dashboard)
        Note this does not always imply that the query should not be executed since
        the results from a hidden query may be used as the input to other queries (SSE etc)
        """
            
        self._internal.hide = hide
    
        return self
    
    def query_type(self, query_type: str) -> typing.Self:    
        """
        Specify the query flavor
        TODO make this required and give it a default
        """
            
        self._internal.query_type = query_type
    
        return self
    
    def alias_by(self, alias_by: str) -> typing.Self:    
        """
        Aliases can be set to modify the legend labels. e.g. {{metric.label.xxx}}. See docs for more detail.
        """
            
        self._internal.alias_by = alias_by
    
        return self
    
    def time_series_list(self, time_series_list: cogbuilder.Builder[googlecloudmonitoring.TimeSeriesList]) -> typing.Self:    
        """
        GCM query type.
        queryType: #QueryType
        Time Series List sub-query properties.
        """
            
        time_series_list_resource = time_series_list.build()
        self._internal.time_series_list = time_series_list_resource
    
        return self
    
    def time_series_query(self, time_series_query: cogbuilder.Builder[googlecloudmonitoring.TimeSeriesQuery]) -> typing.Self:    
        """
        Time Series sub-query properties.
        """
            
        time_series_query_resource = time_series_query.build()
        self._internal.time_series_query = time_series_query_resource
    
        return self
    
    def slo_query(self, slo_query: cogbuilder.Builder[googlecloudmonitoring.SLOQuery]) -> typing.Self:    
        """
        SLO sub-query properties.
        """
            
        slo_query_resource = slo_query.build()
        self._internal.slo_query = slo_query_resource
    
        return self
    
    def prom_ql_query(self, prom_ql_query: cogbuilder.Builder[googlecloudmonitoring.PromQLQuery]) -> typing.Self:    
        """
        PromQL sub-query properties.
        """
            
        prom_ql_query_resource = prom_ql_query.build()
        self._internal.prom_ql_query = prom_ql_query_resource
    
        return self
    
    def datasource(self, datasource: dashboard.DataSourceRef) -> typing.Self:    
        """
        For mixed data sources the selected datasource is on the query level.
        For non mixed scenarios this is undefined.
        TODO find a better way to do this ^ that's friendly to schema
        TODO this shouldn't be unknown but DataSourceRef | null
        """
            
        self._internal.datasource = datasource
    
        return self
    
    def interval_ms(self, interval_ms: float) -> typing.Self:    
        """
        Time interval in milliseconds.
        """
            
        self._internal.interval_ms = interval_ms
    
        return self
    


class TimeSeriesList(cogbuilder.Builder[googlecloudmonitoring.TimeSeriesList]):    
    """
    Time Series List sub-query properties.
    """
    
    _internal: googlecloudmonitoring.TimeSeriesList

    def __init__(self):
        self._internal = googlecloudmonitoring.TimeSeriesList()

    def build(self) -> googlecloudmonitoring.TimeSeriesList:
        """
        Builds the object.
        """
        return self._internal    
    
    def project_name(self, project_name: str) -> typing.Self:    
        """
        GCP project to execute the query against.
        """
            
        self._internal.project_name = project_name
    
        return self
    
    def cross_series_reducer(self, cross_series_reducer: str) -> typing.Self:    
        """
        Reducer applied across a set of time-series values. Defaults to REDUCE_NONE.
        """
            
        self._internal.cross_series_reducer = cross_series_reducer
    
        return self
    
    def alignment_period(self, alignment_period: str) -> typing.Self:    
        """
        Alignment period to use when regularizing data. Defaults to cloud-monitoring-auto.
        """
            
        self._internal.alignment_period = alignment_period
    
        return self
    
    def per_series_aligner(self, per_series_aligner: str) -> typing.Self:    
        """
        Alignment function to be used. Defaults to ALIGN_MEAN.
        """
            
        self._internal.per_series_aligner = per_series_aligner
    
        return self
    
    def group_bys(self, group_bys: list[str]) -> typing.Self:    
        """
        Array of labels to group data by.
        """
            
        self._internal.group_bys = group_bys
    
        return self
    
    def filters(self, filters: list[str]) -> typing.Self:    
        """
        Array of filters to query data by. Labels that can be filtered on are defined by the metric.
        """
            
        self._internal.filters = filters
    
        return self
    
    def view(self, view: str) -> typing.Self:    
        """
        Data view, defaults to FULL.
        """
            
        self._internal.view = view
    
        return self
    
    def title(self, title: str) -> typing.Self:    
        """
        Annotation title.
        """
            
        self._internal.title = title
    
        return self
    
    def text(self, text: str) -> typing.Self:    
        """
        Annotation text.
        """
            
        self._internal.text = text
    
        return self
    
    def secondary_cross_series_reducer(self, secondary_cross_series_reducer: str) -> typing.Self:    
        """
        Only present if a preprocessor is selected. Reducer applied across a set of time-series values. Defaults to REDUCE_NONE.
        """
            
        self._internal.secondary_cross_series_reducer = secondary_cross_series_reducer
    
        return self
    
    def secondary_alignment_period(self, secondary_alignment_period: str) -> typing.Self:    
        """
        Only present if a preprocessor is selected. Alignment period to use when regularizing data. Defaults to cloud-monitoring-auto.
        """
            
        self._internal.secondary_alignment_period = secondary_alignment_period
    
        return self
    
    def secondary_per_series_aligner(self, secondary_per_series_aligner: str) -> typing.Self:    
        """
        Only present if a preprocessor is selected. Alignment function to be used. Defaults to ALIGN_MEAN.
        """
            
        self._internal.secondary_per_series_aligner = secondary_per_series_aligner
    
        return self
    
    def secondary_group_bys(self, secondary_group_bys: list[str]) -> typing.Self:    
        """
        Only present if a preprocessor is selected. Array of labels to group data by.
        """
            
        self._internal.secondary_group_bys = secondary_group_bys
    
        return self
    
    def preprocessor(self, preprocessor: googlecloudmonitoring.PreprocessorType) -> typing.Self:    
        """
        Preprocessor is not part of the API, but is used to store the preprocessor and not affect the UI for the rest of parameters
        """
            
        self._internal.preprocessor = preprocessor
    
        return self
    


class TimeSeriesQuery(cogbuilder.Builder[googlecloudmonitoring.TimeSeriesQuery]):    
    """
    Time Series sub-query properties.
    """
    
    _internal: googlecloudmonitoring.TimeSeriesQuery

    def __init__(self):
        self._internal = googlecloudmonitoring.TimeSeriesQuery()

    def build(self) -> googlecloudmonitoring.TimeSeriesQuery:
        """
        Builds the object.
        """
        return self._internal    
    
    def project_name(self, project_name: str) -> typing.Self:    
        """
        GCP project to execute the query against.
        """
            
        self._internal.project_name = project_name
    
        return self
    
    def query(self, query: str) -> typing.Self:    
        """
        MQL query to be executed.
        """
            
        self._internal.query = query
    
        return self
    
    def graph_period(self, graph_period: str) -> typing.Self:    
        """
        To disable the graphPeriod, it should explictly be set to 'disabled'.
        """
            
        self._internal.graph_period = graph_period
    
        return self
    


class SLOQuery(cogbuilder.Builder[googlecloudmonitoring.SLOQuery]):    
    """
    SLO sub-query properties.
    """
    
    _internal: googlecloudmonitoring.SLOQuery

    def __init__(self):
        self._internal = googlecloudmonitoring.SLOQuery()

    def build(self) -> googlecloudmonitoring.SLOQuery:
        """
        Builds the object.
        """
        return self._internal    
    
    def project_name(self, project_name: str) -> typing.Self:    
        """
        GCP project to execute the query against.
        """
            
        self._internal.project_name = project_name
    
        return self
    
    def per_series_aligner(self, per_series_aligner: str) -> typing.Self:    
        """
        Alignment function to be used. Defaults to ALIGN_MEAN.
        """
            
        self._internal.per_series_aligner = per_series_aligner
    
        return self
    
    def alignment_period(self, alignment_period: str) -> typing.Self:    
        """
        Alignment period to use when regularizing data. Defaults to cloud-monitoring-auto.
        """
            
        self._internal.alignment_period = alignment_period
    
        return self
    
    def selector_name(self, selector_name: str) -> typing.Self:    
        """
        SLO selector.
        """
            
        self._internal.selector_name = selector_name
    
        return self
    
    def service_id(self, service_id: str) -> typing.Self:    
        """
        ID for the service the SLO is in.
        """
            
        self._internal.service_id = service_id
    
        return self
    
    def service_name(self, service_name: str) -> typing.Self:    
        """
        Name for the service the SLO is in.
        """
            
        self._internal.service_name = service_name
    
        return self
    
    def slo_id(self, slo_id: str) -> typing.Self:    
        """
        ID for the SLO.
        """
            
        self._internal.slo_id = slo_id
    
        return self
    
    def slo_name(self, slo_name: str) -> typing.Self:    
        """
        Name of the SLO.
        """
            
        self._internal.slo_name = slo_name
    
        return self
    
    def goal(self, goal: float) -> typing.Self:    
        """
        SLO goal value.
        """
            
        self._internal.goal = goal
    
        return self
    
    def lookback_period(self, lookback_period: str) -> typing.Self:    
        """
        Specific lookback period for the SLO.
        """
            
        self._internal.lookback_period = lookback_period
    
        return self
    


class PromQLQuery(cogbuilder.Builder[googlecloudmonitoring.PromQLQuery]):    
    """
    PromQL sub-query properties.
    """
    
    _internal: googlecloudmonitoring.PromQLQuery

    def __init__(self):
        self._internal = googlecloudmonitoring.PromQLQuery()

    def build(self) -> googlecloudmonitoring.PromQLQuery:
        """
        Builds the object.
        """
        return self._internal    
    
    def project_name(self, project_name: str) -> typing.Self:    
        """
        GCP project to execute the query against.
        """
            
        self._internal.project_name = project_name
    
        return self
    
    def expr(self, expr: str) -> typing.Self:    
        """
        PromQL expression/query to be executed.
        """
            
        self._internal.expr = expr
    
        return self
    
    def step(self, step: str) -> typing.Self:    
        """
        PromQL min step
        """
            
        self._internal.step = step
    
        return self
    


class MetricQuery(cogbuilder.Builder[googlecloudmonitoring.MetricQuery]):    
    """
    @deprecated This type is for migration purposes only. Replaced by TimeSeriesList Metric sub-query properties.
    """
    
    _internal: googlecloudmonitoring.MetricQuery

    def __init__(self):
        self._internal = googlecloudmonitoring.MetricQuery()

    def build(self) -> googlecloudmonitoring.MetricQuery:
        """
        Builds the object.
        """
        return self._internal    
    
    def project_name(self, project_name: str) -> typing.Self:    
        """
        GCP project to execute the query against.
        """
            
        self._internal.project_name = project_name
    
        return self
    
    def per_series_aligner(self, per_series_aligner: str) -> typing.Self:    
        """
        Alignment function to be used. Defaults to ALIGN_MEAN.
        """
            
        self._internal.per_series_aligner = per_series_aligner
    
        return self
    
    def alignment_period(self, alignment_period: str) -> typing.Self:    
        """
        Alignment period to use when regularizing data. Defaults to cloud-monitoring-auto.
        """
            
        self._internal.alignment_period = alignment_period
    
        return self
    
    def alias_by(self, alias_by: str) -> typing.Self:    
        """
        Aliases can be set to modify the legend labels. e.g. {{metric.label.xxx}}. See docs for more detail.
        """
            
        self._internal.alias_by = alias_by
    
        return self
    
    def editor_mode(self, editor_mode: str) -> typing.Self:    
        self._internal.editor_mode = editor_mode
    
        return self
    
    def metric_type(self, metric_type: str) -> typing.Self:    
        self._internal.metric_type = metric_type
    
        return self
    
    def cross_series_reducer(self, cross_series_reducer: str) -> typing.Self:    
        """
        Reducer applied across a set of time-series values. Defaults to REDUCE_NONE.
        """
            
        self._internal.cross_series_reducer = cross_series_reducer
    
        return self
    
    def group_bys(self, group_bys: list[str]) -> typing.Self:    
        """
        Array of labels to group data by.
        """
            
        self._internal.group_bys = group_bys
    
        return self
    
    def filters(self, filters: list[str]) -> typing.Self:    
        """
        Array of filters to query data by. Labels that can be filtered on are defined by the metric.
        """
            
        self._internal.filters = filters
    
        return self
    
    def metric_kind(self, metric_kind: googlecloudmonitoring.MetricKind) -> typing.Self:    
        self._internal.metric_kind = metric_kind
    
        return self
    
    def value_type(self, value_type: str) -> typing.Self:    
        self._internal.value_type = value_type
    
        return self
    
    def view(self, view: str) -> typing.Self:    
        self._internal.view = view
    
        return self
    
    def query(self, query: str) -> typing.Self:    
        """
        MQL query to be executed.
        """
            
        self._internal.query = query
    
        return self
    
    def preprocessor(self, preprocessor: googlecloudmonitoring.PreprocessorType) -> typing.Self:    
        """
        Preprocessor is not part of the API, but is used to store the preprocessor and not affect the UI for the rest of parameters
        """
            
        self._internal.preprocessor = preprocessor
    
        return self
    
    def graph_period(self, graph_period: str) -> typing.Self:    
        """
        To disable the graphPeriod, it should explictly be set to 'disabled'.
        """
            
        self._internal.graph_period = graph_period
    
        return self
    


class LegacyCloudMonitoringAnnotationQuery(cogbuilder.Builder[googlecloudmonitoring.LegacyCloudMonitoringAnnotationQuery]):    
    """
    @deprecated Use TimeSeriesList instead. Legacy annotation query properties for migration purposes.
    """
    
    _internal: googlecloudmonitoring.LegacyCloudMonitoringAnnotationQuery

    def __init__(self):
        self._internal = googlecloudmonitoring.LegacyCloudMonitoringAnnotationQuery()

    def build(self) -> googlecloudmonitoring.LegacyCloudMonitoringAnnotationQuery:
        """
        Builds the object.
        """
        return self._internal    
    
    def project_name(self, project_name: str) -> typing.Self:    
        """
        GCP project to execute the query against.
        """
            
        self._internal.project_name = project_name
    
        return self
    
    def metric_type(self, metric_type: str) -> typing.Self:    
        self._internal.metric_type = metric_type
    
        return self
    
    def ref_id(self, ref_id: str) -> typing.Self:    
        """
        Query refId.
        """
            
        self._internal.ref_id = ref_id
    
        return self
    
    def filters(self, filters: list[str]) -> typing.Self:    
        """
        Array of filters to query data by. Labels that can be filtered on are defined by the metric.
        """
            
        self._internal.filters = filters
    
        return self
    
    def metric_kind(self, metric_kind: googlecloudmonitoring.MetricKind) -> typing.Self:    
        self._internal.metric_kind = metric_kind
    
        return self
    
    def value_type(self, value_type: str) -> typing.Self:    
        self._internal.value_type = value_type
    
        return self
    
    def title(self, title: str) -> typing.Self:    
        """
        Annotation title.
        """
            
        self._internal.title = title
    
        return self
    
    def text(self, text: str) -> typing.Self:    
        """
        Annotation text.
        """
            
        self._internal.text = text
    
        return self
    


class Filter(cogbuilder.Builder[googlecloudmonitoring.Filter]):    
    """
    Query filter representation.
    """
    
    _internal: googlecloudmonitoring.Filter

    def __init__(self):
        self._internal = googlecloudmonitoring.Filter()

    def build(self) -> googlecloudmonitoring.Filter:
        """
        Builds the object.
        """
        return self._internal    
    
    def key(self, key: str) -> typing.Self:    
        """
        Filter key.
        """
            
        self._internal.key = key
    
        return self
    
    def operator(self, operator: str) -> typing.Self:    
        """
        Filter operator.
        """
            
        self._internal.operator = operator
    
        return self
    
    def value(self, value: str) -> typing.Self:    
        """
        Filter value.
        """
            
        self._internal.value = value
    
        return self
    
    def condition(self, condition: str) -> typing.Self:    
        """
        Filter condition.
        """
            
        self._internal.condition = condition
    
        return self
    



# Code generated - EDITING IS FUTILE. DO NOT EDIT.

import enum
import typing
from ..models import common
from ..cog import runtime as cogruntime


class HeatmapColorMode(enum.StrEnum):
    """
    Controls the color mode of the heatmap
    """

    OPACITY = "opacity"
    SCHEME = "scheme"


class HeatmapColorScale(enum.StrEnum):
    """
    Controls the color scale of the heatmap
    """

    LINEAR = "linear"
    EXPONENTIAL = "exponential"


class HeatmapColorOptions:
    """
    Controls various color options
    """

    # Sets the color mode
    mode: typing.Optional['HeatmapColorMode']
    # Controls the color scheme used
    scheme: str
    # Controls the color fill when in opacity mode
    fill: str
    # Controls the color scale
    scale: typing.Optional['HeatmapColorScale']
    # Controls the exponent when scale is set to exponential
    exponent: float
    # Controls the number of color steps
    steps: int
    # Reverses the color scheme
    reverse: bool
    # Sets the minimum value for the color scale
    min_val: typing.Optional[float]
    # Sets the maximum value for the color scale
    max_val: typing.Optional[float]

    def __init__(self, mode: typing.Optional['HeatmapColorMode'] = None, scheme: str = "", fill: str = "", scale: typing.Optional['HeatmapColorScale'] = None, exponent: float = 0, steps: int = 0, reverse: bool = False, min_val: typing.Optional[float] = None, max_val: typing.Optional[float] = None):
        self.mode = mode
        self.scheme = scheme
        self.fill = fill
        self.scale = scale
        self.exponent = exponent
        self.steps = steps
        self.reverse = reverse
        self.min_val = min_val
        self.max_val = max_val

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "scheme": self.scheme,
            "fill": self.fill,
            "exponent": self.exponent,
            "steps": self.steps,
            "reverse": self.reverse,
        }
        if self.mode is not None:
            payload["mode"] = self.mode
        if self.scale is not None:
            payload["scale"] = self.scale
        if self.min_val is not None:
            payload["min"] = self.min_val
        if self.max_val is not None:
            payload["max"] = self.max_val
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "mode" in data:
            args["mode"] = data["mode"]
        if "scheme" in data:
            args["scheme"] = data["scheme"]
        if "fill" in data:
            args["fill"] = data["fill"]
        if "scale" in data:
            args["scale"] = data["scale"]
        if "exponent" in data:
            args["exponent"] = data["exponent"]
        if "steps" in data:
            args["steps"] = data["steps"]
        if "reverse" in data:
            args["reverse"] = data["reverse"]
        if "min" in data:
            args["min_val"] = data["min"]
        if "max" in data:
            args["max_val"] = data["max"]        

        return cls(**args)


class YAxisConfig:
    """
    Configuration options for the yAxis
    """

    # Sets the yAxis unit
    unit: typing.Optional[str]
    # Reverses the yAxis
    reverse: typing.Optional[bool]
    # Controls the number of decimals for yAxis values
    decimals: typing.Optional[float]
    # Sets the minimum value for the yAxis
    min_val: typing.Optional[float]
    axis_placement: typing.Optional[common.AxisPlacement]
    axis_color_mode: typing.Optional[common.AxisColorMode]
    axis_label: typing.Optional[str]
    axis_width: typing.Optional[float]
    axis_soft_min: typing.Optional[float]
    axis_soft_max: typing.Optional[float]
    axis_grid_show: typing.Optional[bool]
    scale_distribution: typing.Optional[common.ScaleDistributionConfig]
    axis_centered_zero: typing.Optional[bool]
    # Sets the maximum value for the yAxis
    max_val: typing.Optional[float]
    axis_border_show: typing.Optional[bool]

    def __init__(self, unit: typing.Optional[str] = None, reverse: typing.Optional[bool] = None, decimals: typing.Optional[float] = None, min_val: typing.Optional[float] = None, axis_placement: typing.Optional[common.AxisPlacement] = None, axis_color_mode: typing.Optional[common.AxisColorMode] = None, axis_label: typing.Optional[str] = None, axis_width: typing.Optional[float] = None, axis_soft_min: typing.Optional[float] = None, axis_soft_max: typing.Optional[float] = None, axis_grid_show: typing.Optional[bool] = None, scale_distribution: typing.Optional[common.ScaleDistributionConfig] = None, axis_centered_zero: typing.Optional[bool] = None, max_val: typing.Optional[float] = None, axis_border_show: typing.Optional[bool] = None):
        self.unit = unit
        self.reverse = reverse
        self.decimals = decimals
        self.min_val = min_val
        self.axis_placement = axis_placement
        self.axis_color_mode = axis_color_mode
        self.axis_label = axis_label
        self.axis_width = axis_width
        self.axis_soft_min = axis_soft_min
        self.axis_soft_max = axis_soft_max
        self.axis_grid_show = axis_grid_show
        self.scale_distribution = scale_distribution
        self.axis_centered_zero = axis_centered_zero
        self.max_val = max_val
        self.axis_border_show = axis_border_show

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.unit is not None:
            payload["unit"] = self.unit
        if self.reverse is not None:
            payload["reverse"] = self.reverse
        if self.decimals is not None:
            payload["decimals"] = self.decimals
        if self.min_val is not None:
            payload["min"] = self.min_val
        if self.axis_placement is not None:
            payload["axisPlacement"] = self.axis_placement
        if self.axis_color_mode is not None:
            payload["axisColorMode"] = self.axis_color_mode
        if self.axis_label is not None:
            payload["axisLabel"] = self.axis_label
        if self.axis_width is not None:
            payload["axisWidth"] = self.axis_width
        if self.axis_soft_min is not None:
            payload["axisSoftMin"] = self.axis_soft_min
        if self.axis_soft_max is not None:
            payload["axisSoftMax"] = self.axis_soft_max
        if self.axis_grid_show is not None:
            payload["axisGridShow"] = self.axis_grid_show
        if self.scale_distribution is not None:
            payload["scaleDistribution"] = self.scale_distribution
        if self.axis_centered_zero is not None:
            payload["axisCenteredZero"] = self.axis_centered_zero
        if self.max_val is not None:
            payload["max"] = self.max_val
        if self.axis_border_show is not None:
            payload["axisBorderShow"] = self.axis_border_show
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "unit" in data:
            args["unit"] = data["unit"]
        if "reverse" in data:
            args["reverse"] = data["reverse"]
        if "decimals" in data:
            args["decimals"] = data["decimals"]
        if "min" in data:
            args["min_val"] = data["min"]
        if "axisPlacement" in data:
            args["axis_placement"] = data["axisPlacement"]
        if "axisColorMode" in data:
            args["axis_color_mode"] = data["axisColorMode"]
        if "axisLabel" in data:
            args["axis_label"] = data["axisLabel"]
        if "axisWidth" in data:
            args["axis_width"] = data["axisWidth"]
        if "axisSoftMin" in data:
            args["axis_soft_min"] = data["axisSoftMin"]
        if "axisSoftMax" in data:
            args["axis_soft_max"] = data["axisSoftMax"]
        if "axisGridShow" in data:
            args["axis_grid_show"] = data["axisGridShow"]
        if "scaleDistribution" in data:
            args["scale_distribution"] = common.ScaleDistributionConfig.from_json(data["scaleDistribution"])
        if "axisCenteredZero" in data:
            args["axis_centered_zero"] = data["axisCenteredZero"]
        if "max" in data:
            args["max_val"] = data["max"]
        if "axisBorderShow" in data:
            args["axis_border_show"] = data["axisBorderShow"]        

        return cls(**args)


class CellValues:
    """
    Controls cell value options
    """

    # Controls the cell value unit
    unit: typing.Optional[str]
    # Controls the number of decimals for cell values
    decimals: typing.Optional[float]

    def __init__(self, unit: typing.Optional[str] = None, decimals: typing.Optional[float] = None):
        self.unit = unit
        self.decimals = decimals

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.unit is not None:
            payload["unit"] = self.unit
        if self.decimals is not None:
            payload["decimals"] = self.decimals
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "unit" in data:
            args["unit"] = data["unit"]
        if "decimals" in data:
            args["decimals"] = data["decimals"]        

        return cls(**args)


class FilterValueRange:
    """
    Controls the value filter range
    """

    # Sets the filter range to values less than or equal to the given value
    le: typing.Optional[float]
    # Sets the filter range to values greater than or equal to the given value
    ge: typing.Optional[float]

    def __init__(self, le: typing.Optional[float] = None, ge: typing.Optional[float] = None):
        self.le = le
        self.ge = ge

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.le is not None:
            payload["le"] = self.le
        if self.ge is not None:
            payload["ge"] = self.ge
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "le" in data:
            args["le"] = data["le"]
        if "ge" in data:
            args["ge"] = data["ge"]        

        return cls(**args)


class HeatmapTooltip:
    """
    Controls tooltip options
    """

    # Controls if the tooltip is shown
    show: bool
    # Controls if the tooltip shows a histogram of the y-axis values
    y_histogram: typing.Optional[bool]
    # Controls if the tooltip shows a color scale in header
    show_color_scale: typing.Optional[bool]

    def __init__(self, show: bool = False, y_histogram: typing.Optional[bool] = None, show_color_scale: typing.Optional[bool] = None):
        self.show = show
        self.y_histogram = y_histogram
        self.show_color_scale = show_color_scale

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "show": self.show,
        }
        if self.y_histogram is not None:
            payload["yHistogram"] = self.y_histogram
        if self.show_color_scale is not None:
            payload["showColorScale"] = self.show_color_scale
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "show" in data:
            args["show"] = data["show"]
        if "yHistogram" in data:
            args["y_histogram"] = data["yHistogram"]
        if "showColorScale" in data:
            args["show_color_scale"] = data["showColorScale"]        

        return cls(**args)


class HeatmapLegend:
    """
    Controls legend options
    """

    # Controls if the legend is shown
    show: bool

    def __init__(self, show: bool = False):
        self.show = show

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "show": self.show,
        }
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "show" in data:
            args["show"] = data["show"]        

        return cls(**args)


class ExemplarConfig:
    """
    Controls exemplar options
    """

    # Sets the color of the exemplar markers
    color: str

    def __init__(self, color: str = ""):
        self.color = color

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "color": self.color,
        }
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "color" in data:
            args["color"] = data["color"]        

        return cls(**args)


class RowsHeatmapOptions:
    """
    Controls frame rows options
    """

    # Sets the name of the cell when not calculating from data
    value: typing.Optional[str]
    # Controls tick alignment when not calculating from data
    layout: typing.Optional[common.HeatmapCellLayout]

    def __init__(self, value: typing.Optional[str] = None, layout: typing.Optional[common.HeatmapCellLayout] = None):
        self.value = value
        self.layout = layout

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.value is not None:
            payload["value"] = self.value
        if self.layout is not None:
            payload["layout"] = self.layout
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "value" in data:
            args["value"] = data["value"]
        if "layout" in data:
            args["layout"] = data["layout"]        

        return cls(**args)


class Options:
    # Controls if the heatmap should be calculated from data
    calculate: typing.Optional[bool]
    # Calculation options for the heatmap
    calculation: typing.Optional[common.HeatmapCalculationOptions]
    # Controls the color options
    color: 'HeatmapColorOptions'
    # Filters values between a given range
    filter_values: typing.Optional['FilterValueRange']
    # Controls tick alignment and value name when not calculating from data
    rows_frame: typing.Optional['RowsHeatmapOptions']
    # | *{
    # 	layout: ui.HeatmapCellLayout & "auto" // TODO: fix after remove when https://github.com/grafana/cuetsy/issues/74 is fixed
    # }
    # Controls the display of the value in the cell
    show_value: common.VisibilityMode
    # Controls gap between cells
    cell_gap: typing.Optional[int]
    # Controls cell radius
    cell_radius: typing.Optional[float]
    # Controls cell value unit
    cell_values: typing.Optional['CellValues']
    # Controls yAxis placement
    y_axis: 'YAxisConfig'
    # | *{
    # 	axisPlacement: ui.AxisPlacement & "left" // TODO: fix after remove when https://github.com/grafana/cuetsy/issues/74 is fixed
    # }
    # Controls legend options
    legend: 'HeatmapLegend'
    # Controls tooltip options
    tooltip: 'HeatmapTooltip'
    # Controls exemplar options
    exemplars: 'ExemplarConfig'

    def __init__(self, calculate: typing.Optional[bool] = False, calculation: typing.Optional[common.HeatmapCalculationOptions] = None, color: typing.Optional['HeatmapColorOptions'] = None, filter_values: typing.Optional['FilterValueRange'] = None, rows_frame: typing.Optional['RowsHeatmapOptions'] = None, show_value: typing.Optional[common.VisibilityMode] = None, cell_gap: typing.Optional[int] = 1, cell_radius: typing.Optional[float] = None, cell_values: typing.Optional['CellValues'] = None, y_axis: typing.Optional['YAxisConfig'] = None, legend: typing.Optional['HeatmapLegend'] = None, tooltip: typing.Optional['HeatmapTooltip'] = None, exemplars: typing.Optional['ExemplarConfig'] = None):
        self.calculate = calculate
        self.calculation = calculation
        self.color = color if color is not None else HeatmapColorOptions(exponent=0.5, fill="dark-orange", reverse=False, scheme="Oranges", steps=64)
        self.filter_values = filter_values if filter_values is not None else FilterValueRange(le=1e-09)
        self.rows_frame = rows_frame
        self.show_value = show_value if show_value is not None else common.VisibilityMode.AUTO
        self.cell_gap = cell_gap
        self.cell_radius = cell_radius
        self.cell_values = cell_values
        self.y_axis = y_axis if y_axis is not None else YAxisConfig()
        self.legend = legend if legend is not None else HeatmapLegend(show=True)
        self.tooltip = tooltip if tooltip is not None else HeatmapTooltip()
        self.exemplars = exemplars if exemplars is not None else ExemplarConfig(color="rgba(255,0,255,0.7)")

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "color": self.color,
            "showValue": self.show_value,
            "yAxis": self.y_axis,
            "legend": self.legend,
            "tooltip": self.tooltip,
            "exemplars": self.exemplars,
        }
        if self.calculate is not None:
            payload["calculate"] = self.calculate
        if self.calculation is not None:
            payload["calculation"] = self.calculation
        if self.filter_values is not None:
            payload["filterValues"] = self.filter_values
        if self.rows_frame is not None:
            payload["rowsFrame"] = self.rows_frame
        if self.cell_gap is not None:
            payload["cellGap"] = self.cell_gap
        if self.cell_radius is not None:
            payload["cellRadius"] = self.cell_radius
        if self.cell_values is not None:
            payload["cellValues"] = self.cell_values
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "calculate" in data:
            args["calculate"] = data["calculate"]
        if "calculation" in data:
            args["calculation"] = common.HeatmapCalculationOptions.from_json(data["calculation"])
        if "color" in data:
            args["color"] = HeatmapColorOptions.from_json(data["color"])
        if "filterValues" in data:
            args["filter_values"] = FilterValueRange.from_json(data["filterValues"])
        if "rowsFrame" in data:
            args["rows_frame"] = RowsHeatmapOptions.from_json(data["rowsFrame"])
        if "showValue" in data:
            args["show_value"] = data["showValue"]
        if "cellGap" in data:
            args["cell_gap"] = data["cellGap"]
        if "cellRadius" in data:
            args["cell_radius"] = data["cellRadius"]
        if "cellValues" in data:
            args["cell_values"] = CellValues.from_json(data["cellValues"])
        if "yAxis" in data:
            args["y_axis"] = YAxisConfig.from_json(data["yAxis"])
        if "legend" in data:
            args["legend"] = HeatmapLegend.from_json(data["legend"])
        if "tooltip" in data:
            args["tooltip"] = HeatmapTooltip.from_json(data["tooltip"])
        if "exemplars" in data:
            args["exemplars"] = ExemplarConfig.from_json(data["exemplars"])        

        return cls(**args)


class FieldConfig:
    scale_distribution: typing.Optional[common.ScaleDistributionConfig]
    hide_from: typing.Optional[common.HideSeriesConfig]

    def __init__(self, scale_distribution: typing.Optional[common.ScaleDistributionConfig] = None, hide_from: typing.Optional[common.HideSeriesConfig] = None):
        self.scale_distribution = scale_distribution
        self.hide_from = hide_from

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
        }
        if self.scale_distribution is not None:
            payload["scaleDistribution"] = self.scale_distribution
        if self.hide_from is not None:
            payload["hideFrom"] = self.hide_from
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "scaleDistribution" in data:
            args["scale_distribution"] = common.ScaleDistributionConfig.from_json(data["scaleDistribution"])
        if "hideFrom" in data:
            args["hide_from"] = common.HideSeriesConfig.from_json(data["hideFrom"])        

        return cls(**args)





def variant_config() -> cogruntime.PanelCfgConfig:
    return cogruntime.PanelCfgConfig(
        identifier="heatmap",
        options_from_json_hook=Options.from_json,
        field_config_from_json_hook=FieldConfig.from_json,
    )




# Code generated - EDITING IS FUTILE. DO NOT EDIT.

import typing
from ..models import common
from ..cog import runtime as cogruntime


class Options:
    # Represents the index of the selected frame
    frame_index: float
    # Controls whether the panel should show the header
    show_header: bool
    # Controls whether the header should show icons for the column types
    show_type_icons: typing.Optional[bool]
    # Used to control row sorting
    sort_by: typing.Optional[list[common.TableSortByFieldState]]
    # Controls footer options
    footer: typing.Optional[common.TableFooterOptions]
    # Controls the height of the rows
    cell_height: typing.Optional[common.TableCellHeight]

    def __init__(self, frame_index: float = 0, show_header: bool = True, show_type_icons: typing.Optional[bool] = False, sort_by: typing.Optional[list[common.TableSortByFieldState]] = None, footer: typing.Optional[common.TableFooterOptions] = None, cell_height: typing.Optional[common.TableCellHeight] = None):
        self.frame_index = frame_index
        self.show_header = show_header
        self.show_type_icons = show_type_icons
        self.sort_by = sort_by
        self.footer = footer if footer is not None else common.TableFooterOptions(count_rows=False, reducer=None, show=False)
        self.cell_height = cell_height if cell_height is not None else common.TableCellHeight.SM

    def to_json(self) -> dict[str, object]:
        payload: dict[str, object] = {
            "frameIndex": self.frame_index,
            "showHeader": self.show_header,
        }
        if self.show_type_icons is not None:
            payload["showTypeIcons"] = self.show_type_icons
        if self.sort_by is not None:
            payload["sortBy"] = self.sort_by
        if self.footer is not None:
            payload["footer"] = self.footer
        if self.cell_height is not None:
            payload["cellHeight"] = self.cell_height
        return payload

    @classmethod
    def from_json(cls, data: dict[str, typing.Any]) -> typing.Self:
        args: dict[str, typing.Any] = {}
        
        if "frameIndex" in data:
            args["frame_index"] = data["frameIndex"]
        if "showHeader" in data:
            args["show_header"] = data["showHeader"]
        if "showTypeIcons" in data:
            args["show_type_icons"] = data["showTypeIcons"]
        if "sortBy" in data:
            args["sort_by"] = [common.TableSortByFieldState.from_json(item) for item in data["sortBy"]]
        if "footer" in data:
            args["footer"] = common.TableFooterOptions.from_json(data["footer"])
        if "cellHeight" in data:
            args["cell_height"] = data["cellHeight"]        

        return cls(**args)


FieldConfig: typing.TypeAlias = common.TableFieldOptions





def variant_config() -> cogruntime.PanelCfgConfig:
    return cogruntime.PanelCfgConfig(
        identifier="table",
        options_from_json_hook=Options.from_json,
        field_config_from_json_hook=FieldConfig.from_json,
    )

